// Copyright (C) 2006-2009 Simula Research Laboratory and Johan Hake
// Licensed under the GNU GPL Version 3.0.

#ifndef PLC_IS_INCLUDED
#define PLC_IS_INCLUDED
/**
 * @file plc.h
 * @brief A wraper for a tetgenio object used to represent a Piecewise Linear Complex
 *
 * @author Johan Hake
 * @date 2007-07-17
 */

#include <tetgen.h>
#include <string>

#include "tetmesh.h"

namespace tritetmesh{

  class PLC 
  {

  public:
  
    /// Constructor
    PLC();
  
    /// Destructor
    ~PLC();

    /// Use tetgen to detect self-intersections of facets of the PLC.
    void test(bool save_test_result = false);
  
    /// Load plc from file. Accepted file formats are: poly, smesh, off, ply, stl, mesh
    void load(std::string file_name);
  
    /// Write plc to file 
    void save_tetgen(std::string file_name);

    /// Clear the tetgenio object from any previous registered data
    void clear();
  
    /// Assignment operator
    const tritetmesh::PLC& operator=(const tritetmesh::PLC& plc);

    /// Print the present tetgen parameters
    void print_tetgen_parameters();

    /// Set quality of generated mesh.
    inline void set_radius_edge_ratio(double ratio){radius_edge_ratio = ratio;}

    /// Set uniform volume constraint.
    inline void set_uniform_volume_constraint(double constraint)
    {
      uniform_volume_constraint = constraint;
    }

    /// Set verbosity
    inline void set_verbosity(int verbosity){this->verbosity = verbosity;}

    /// Supress splitting of facets
    inline void set_splitting_of_facets(bool split){split_facets = split;}

    /// Build a mesh from the information in the plc
    void build_mesh(tritetmesh::TetMesh &output, std::string switches);
    void build_mesh(tritetmesh::TetMesh &output);

  private:
    friend class PLCEditor;
  
    double radius_edge_ratio;
    double uniform_volume_constraint;
    int    verbosity;
    bool   split_facets;             
    bool   use_regional_attributes;
    bool   use_regional_volume_constraints;
    tetgen::tetgenio data;
  
  };

}

#endif
