// Copyright (C) 2006-2009 Simula Research Laboratory and Johan Hake
// Licensed under the GNU GPL Version 3.0.

#ifndef TETMESH_IS_INCLUDED
#define TETMESH_IS_INCLUDED
/**
 * @file tetmesh.h
 * @brief A wraper for a tetgenio object used to represent a tetgen mesh
 *
 * @author Johan Hake
 * @date 2007-07-17
 */

#include <tetgen_mesh.h>
#include <string>

namespace tritetmesh{

  class TetMesh
  {

  public:
    
    /// Constructor
    TetMesh();
    
    /// Destructor
    ~TetMesh();
    
    /// Load mesh from file.
    /// file_name is the inputfile with outsuffix.
    /// Need at least an .ele and a .node file, if an .face and/or a .vol file exsist,
    /// they will also be read.
    void load_tetgen(std::string file_name);
    
    /// Write the mesh to a set of tetgen files
    void save_tetgen(std::string file_name);
    
    /// Write the mesh to a set of dolfin files
    void save_dolfin(std::string file_name);
    
    /// Write the mesh to a .mesh file that can be read by medit
    void save_medit(std::string file_name);
    
    /// Write the mesh to a .msh file that can be read by Gid
    void save_gid(std::string file_name);
    
    /// Write the mesh to a .msh file that can be read by Geomview
    void save_off(std::string file_name);
    
    /// Check consistency of the mesh
    void check_consistency();
    
    /// Clear the tetgenio object from any previous registered data
    void clear();
    
    /// Assignment operator
    const tritetmesh::TetMesh& operator=(const tritetmesh::TetMesh& tetmesh);
    
    /// Function to refine a mesh 
    /// TODO: implement
    void refine_mesh(tritetmesh::TetMesh &output, std::string switches);

    /// Print tetgen help
    inline void print_tetgen_help(){tetgen_behavior.usage();}
    
  private:
  
    // PLC needs us to be friend with him, so he can be able to produce me and this also goes for the TetMeshConverter
    friend class PLC;
    friend class TriTetMeshConverter;
  
    // the tetgenio object holding the data of the mesh.
    tetgen::tetgenio data;

    // tetgenbehavior object, used to reconstruct the mesh using tetgenmesh class from tetgen
    tetgen::tetgenbehavior tetgen_behavior;
  
    // return a reconstructed tetgenmesh, used to save the tetmesh in other formats provided by tetgen.
    tetgen::tetgenmesh *reconstruct_mesh();
  
  };

}

#endif

