// Copyright (C) 2006-2009 Simula Research Laboratory and Johan Hake
// Licensed under the GNU GPL Version 3.0.

#ifndef PSLG_IS_INCLUDED
#define PSLG_IS_INCLUDED
/**
 * @file pslg.h
 * @brief A wraper for a triangulateio object used to represent a Planar Straight Line Graph
 *
 * @author Johan Hake
 * @date 2007-09-21
 */

extern "C" {
#include <triangle_mesh.h>
}

#include "trimesh.h"
#include "areafunction.h"

namespace tritetmesh{
  
  class PSLG 
  {
    
  public:
    
    /// Maximum number of characters in a file name (including the null).  
    enum {FILENAMESIZE = 1024};
    
    /// Constructor
    PSLG();
    
    /// Destructor
    ~PSLG();
    
    /// Clear the triangulateio object from any previous registered data and initialize it
    inline void clear(){deinitialize(); initialize();}
    
    /// Save the mesh in a triangle format
    void save_triangle(char* file_name);
    
    /// Assignment operator
    const PSLG& operator=(const PSLG& pslg);
    
    /// Print the present triangle parameters
    void print_triangle_parameters();
    
    /// Set a minimum angle of a generated triangle.  (triangle -qX option)
    inline void set_minimum_angle(double angle){minimum_angle = angle;}
    
    /// Set uniform volume constraint  (triangle -aX.X option)
    inline void set_uniform_area_constraint(double constraint)
    {
      uniform_area_constraint = constraint;
    }
    
    /// Set a user defined area function.
    inline void add_area_function(tritetmesh::AreaFunction &area_func)
    {
      this->area_func = &area_func;
    }
    
    /// Set verbosity  (triangle -V option)
    inline void set_verbosity(int verbosity){this->verbosity = verbosity;}
    
    /// set generate edge indformation  (triangle -e option)
    inline void set_generate_edge_information(bool generate)
    {
      generate_edge_information = generate;
    }

    /// Enclose a convex hull (triangle -c option)
    inline void set_enclose_convex_hull(bool enclose)
    {
      enclose_convex_hull = enclose;
    }

    /// Build a mesh from the information in the pslg
    void build_mesh(tritetmesh::TriMesh &output, char* switches);
    void build_mesh(tritetmesh::TriMesh &output);

  private:
    friend class PSLGEditor;
    
    void initialize();
    void deinitialize();
    void generate(TriMesh &output,char* switches);
    
    double minimum_angle;
    double uniform_area_constraint;
    int    verbosity;
    bool   use_regional_attributes;
    bool   use_regional_area_constraints;
    bool   generate_edge_information;
    bool   enclose_convex_hull;     
     
    tritetmesh::AreaFunction *area_func;
    
    triangulateio data;
  
  };

}

#endif
