// Copyright (C) 2006-2009 Simula Research Laboratory and Johan Hake
// Licensed under the GNU GPL Version 3.0.

#ifndef TRIMESH_IS_INCLUDED
#define TRIMESH_IS_INCLUDED
/**
 * @file trimesh.h
 * @brief A wraper for a triangulateio object used to represent a triangle mesh
 *
 * @author Johan Hake
 * @date 2007-09-21
 */

#include <string>

extern "C" {
#include <triangle_mesh.h>
}

#include "areafunction.h"

namespace tritetmesh{

  class TriMesh 
  {

  public:
  
    /// Maximum number of characters in a file name (including the null).  
    enum {FILENAMESIZE = 1024};

    /// Constructor
    TriMesh();
  
    /// Destructor
    ~TriMesh();

    /// Clear the triangulateio object from any previous registered data and initialize it
    inline void clear(){deinitialize(); initialize();}
  
    /// Save the mesh in a triangle format
    void save_triangle(char* file_name);

    /// Save the mesh in a dolfin format
    void save_dolfin(std::string file_name);

    /// Assignment operator
    const tritetmesh::TriMesh& operator=(const tritetmesh::TriMesh& trimesh);

    /// Print the present triangle parameters
    void print_triangle_parameters();

    /// Set quality of generated mesh.
    inline void set_minimum_angle(double angle){minimum_angle = angle;}

    /// Set uniform volume constraint.
    inline void set_uniform_area_constraint(double constraint)
    {
      uniform_area_constraint = constraint;
    }
  
    /// Set a user defined area function.
    inline void set_area_function(tritetmesh::AreaFunction &area_func)
    {
      this->area_func = &area_func;
    }
  
    /// Refine a mesh
    void refine_mesh(tritetmesh::TriMesh &output, char* switches);
    void refine_mesh(tritetmesh::TriMesh &output);

  private:
    friend class PSLG;
    friend class TriTetMeshConverter;
  
    void initialize();
    void deinitialize();

    double minimum_angle;
    double uniform_area_constraint;
    int    verbosity;
    tritetmesh::AreaFunction *area_func;
    
    triangulateio data;
  
  };

}

#endif
